<!DOCTYPE html>
<html>
    <head>
        <meta charset="utf-8" />
        <title>Please verify new e-mail</title>
    </head>
    <body>
        <p>
            You can verify the email {{ email }} by clicking
            <a href="{{ verification_url }}">here</a>.
        </p>
    </body>
</html>
